/*!
 * @asigloo/vue-dynamic-forms v0.0.0-development
 * (c) 2020 Alvaro Saburido
 * @license MIT
 */
var asVueDynamicForms = (function (exports, vue) {
  'use strict';

  const dynamicFormsSymbol = Symbol();
  function useDynamicForms() {
      const dynamicForms = vue.inject(dynamicFormsSymbol);
      if (!dynamicForms)
          throw new Error('No dynamicForms provided!!!');
      return dynamicForms;
  }

  function useInputEvents(control, emit) {
      function onChange($event) {
          if (control) {
              control.value = $event.target.value;
              control.dirty = true;
          }
          emit('changed', $event.target.value);
      }
      function onFocus() {
          emit('focus');
      }
      function onBlur() {
          emit('blur');
          if (control) {
              control.touched = true;
          }
      }
      return {
          onFocus,
          onChange,
          onBlur,
      };
  }

  const props = {
      control: Object,
  };
  var script = vue.defineComponent({
      name: 'asTextInput',
      props,
      setup(props, { emit }) {
          const { onChange, onFocus, onBlur } = useInputEvents(props?.control, emit);
          return () => vue.h('input', {
              name: props?.control?.name || '',
              type: props?.control?.type,
              disabled: props?.control?.disabled,
              placeholder: props?.control?.placeholder,
              class: ['form-control'],
              value: props?.control?.value,
              onFocus,
              onBlur,
              onChange,
          });
      },
  });

  const render = () => {};


  script.render = render;
  script.__file = "src/components/text-input/TextInput.vue";

  const props$1 = {
      control: Object,
  };
  var script$1 = vue.defineComponent({
      name: 'asSelectInput',
      props: props$1,
      setup(props, { emit }) {
          return () => {
              const { onChange, onFocus, onBlur } = useInputEvents(props?.control, emit);
              const options = props?.control?.options?.map(({ key, value, disabled }) => vue.h('option', { key, value, disabled }, value));
              return vue.h('select', {
                  name: props?.control?.name || '',
                  disabled: props?.control?.disabled,
                  placeholder: props?.control?.placeholder,
                  class: ['form-control'],
                  value: props?.control?.value,
                  onFocus,
                  onBlur,
                  onChange,
              }, options);
          };
      },
  });

  const render$1 = () => {};


  script$1.render = render$1;
  script$1.__file = "src/components/select-input/SelectInput.vue";

  const props$2 = {
      control: Object,
  };
  var script$2 = vue.defineComponent({
      name: 'asTextAreaInput',
      props: props$2,
      setup(props, { emit }) {
          const { onChange, onFocus, onBlur } = useInputEvents(props?.control, emit);
          return () => vue.h('textarea', {
              name: props?.control?.name || '',
              disabled: props?.control?.disabled,
              placeholder: props?.control?.placeholder,
              class: ['form-control'],
              value: props?.control?.value,
              rows: props?.control?.rows,
              cols: props?.control?.cols,
              onFocus,
              onBlur,
              onChange,
          });
      },
  });

  const render$2 = () => {};


  script$2.render = render$2;
  script$2.__file = "src/components/text-area-input/TextAreaInput.vue";

  const { assign, entries, values, keys } = Object;
  const slugify = (text) => text
      .toString()
      .toLowerCase()
      .replace(/\s+/g, '-') // Replace spaces with -
      //eslint-disable-next-line
      .replace(/[^\w\-]+/g, '') // Remove all non-word chars
      .replace(/--+/g, '-') // Replace multiple - with single -
      .replace(/^-+/, '') // Trim - from start of text
      .replace(/-+$/, '');
  const isArray = (a) => !!a && a.constructor === Array;
  const isObject = (a) => !!a && a.constructor === Object;
  const isEmpty = (entry) => {
      if (isArray(entry)) {
          return entry.length > 0;
      }
      if (isObject(entry)) {
          return entries(entry).length > 0;
      }
  };

  /* eslint-disable @typescript-eslint/no-use-before-define */
  const components = {
      TextInput: script,
      SelectInput: script$1,
      TextAreaInput: script$2,
  };
  const props$3 = {
      control: {
          type: Object,
          required: true,
      },
  };
  var script$3 = vue.defineComponent({
      name: 'asDynamicInput',
      components,
      props: props$3,
      setup(props, { emit }) {
          let component;
          const getClasses = vue.computed(() => {
              return [
                  'dynamic-input',
                  'form-group',
                  {
                      'form-group--error': showErrors.value,
                  },
                  `${props?.control?.customClass || ''}`,
              ];
          });
          const showErrors = vue.computed(() => {
              return props?.control?.errors && keys(props?.control?.errors).length > 0;
              /* props.control.errors &&
                Object.keys(props.control.errors).length > 0 &&
                (this.submited || this.autoValidate) */
          });
          const errorMessages = vue.computed(() => {
              const errors = values(props?.control?.errors || {});
              if (errors.length > 0) {
                  return errors.map(value => value.text);
              }
              return [];
          });
          function validate() {
              if (props.control &&
                  props.control.validations &&
                  isEmpty(props.control.validations)) {
                  const validation = props.control.validations.reduce((prev, curr) => {
                      const val = typeof curr.validator === 'function'
                          ? curr.validator(props.control)
                          : null;
                      if (val !== null) {
                          const [key, value] = entries(val)[0];
                          const obj = {};
                          obj[key] = {
                              value,
                              text: curr.text,
                          };
                          return {
                              ...prev,
                              ...obj,
                          };
                      }
                      return {
                          ...prev,
                      };
                  }, {});
                  props.control.errors = validation;
                  props.control.valid = Object.keys(validation).length === 0;
                  console.log(props.control);
              }
          }
          function valueChange(value) {
              const newValue = {};
              if (props.control) {
                  newValue[props.control.name] = value;
                  validate();
                  emit('changed', newValue);
              }
          }
          const attributes = vue.computed(() => {
              return {
                  control: props.control,
                  onChanged: valueChange,
              };
          });
          return () => {
              switch (props?.control?.type) {
                  case 'text':
                  case 'email':
                  case 'password':
                  case 'url':
                      component = vue.h(script, attributes.value);
                      break;
                  case 'select':
                      component = vue.h(script$1, attributes.value);
                      break;
                  case 'textarea':
                      component = vue.h(script$2, attributes.value);
                      break;
              }
              return vue.h('div', {
                  class: getClasses.value,
              }, [
                  vue.h('label', {
                      class: 'form-label',
                      for: props?.control?.label,
                  }, props?.control?.label),
                  component,
                  vue.h('div', {
                      class: 'form-errors',
                  }, errorMessages.value.map(error => vue.h('p', { class: 'form-error' }, error))),
              ]);
          };
      },
  });

  const render$3 = () => {};


  script$3.render = render$3;
  script$3.__file = "src/components/dynamic-input/DynamicInput.vue";

  class FormValidation {
      constructor(validator, text) {
          this.validator = validator;
          this.text = text || '';
      }
  }
  class InputBase {
      constructor(options = {}) {
          this.value = options.value;
          this.name = options.name || slugify(options.label || '') || '';
          this.label = options.label || '';
          this.disabled = !!options.disabled;
          this.order = options.order === undefined ? 1 : options.order;
          this.type = options.type || '';
          this.customClass = options.customClass;
          this.placeholder = options.placeholder;
          this.validations = options.validations || [];
          this.options = options.options;
          this.rows = options.rows || 0;
          this.cols = options.cols || 0;
      }
  }
  class TextInput extends InputBase {
      constructor() {
          super(...arguments);
          this.type = 'text';
      }
  }
  class EmailInput extends InputBase {
      constructor() {
          super(...arguments);
          this.type = 'email';
      }
  }
  class PasswordInput extends InputBase {
      constructor() {
          super(...arguments);
          this.type = 'password';
      }
  }
  class SelectInput extends InputBase {
      constructor() {
          super(...arguments);
          this.type = 'select';
      }
  }
  class TextAreaInput extends InputBase {
      constructor() {
          super(...arguments);
          this.type = 'textarea';
      }
  }
  class FormControl extends InputBase {
      constructor(options = {}) {
          super({
              value: options.value,
              name: options.name,
              label: options.label,
              type: options.type,
              disabled: !!options.disabled,
              order: options.order,
              customClass: options.customClass,
              validations: options.validations,
              options: options.options,
              rows: options.rows,
              cols: options.cols,
          });
          this.valid = true;
          this.invalid = false;
          this.dirty = false;
          this.touched = false;
          this.errors = {};
      }
  }

  const props$4 = {
      form: {
          type: Object,
          required: true,
      },
  };
  const components$1 = {
      DynamicInput: script$3,
  };
  var script$4 = vue.defineComponent({
      name: 'asDynamicForm',
      props: props$4,
      components: components$1,
      setup(props, { emit }) {
          const controls = vue.ref([]);
          const formValues = vue.reactive({});
          const submited = vue.ref(false);
          vue.onMounted(() => {
              mapControls();
              initValues();
          });
          const isValid = vue.computed(() => {
              const control = controls?.value?.find(control => !control.valid);
              return control ? control.valid : true;
          });
          const errors = vue.computed(() => {
              return controls.value
                  ? controls.value.reduce((prev, curr) => {
                      const errors = Object.keys(curr.errors || {}) || [];
                      if (errors.length > 0) {
                          const error = {};
                          error[curr.name] = curr.errors;
                          return {
                              ...prev,
                              ...error,
                          };
                      }
                      return prev;
                  }, {})
                  : {};
          });
          function valueChange(changedValue) {
              Object.assign(formValues, changedValue);
              emit('changed', formValues);
          }
          function mapControls(empty) {
              controls.value =
                  props.form?.fields?.map((field) => empty
                      ? new FormControl({ ...field, value: null })
                      : new FormControl({ ...field })) || [];
          }
          function resetForm() {
              mapControls(true);
          }
          function handleSubmit() {
              submited.value = true;
              if (isValid.value) {
                  emit('submited', formValues);
                  resetForm();
              }
              else {
                  emit('error', formValues);
              }
          }
          function initValues() {
              Object.assign(formValues, controls.value
                  ? controls.value.reduce((prev, curr) => {
                      const obj = {};
                      obj[curr.name] =
                          curr.type === 'number' ? parseFloat(curr.value) : curr.value;
                      return {
                          ...prev,
                          ...obj,
                      };
                  }, {})
                  : {});
              emit('changed', formValues);
          }
          vue.watch(props, () => {
              mapControls();
          });
          return {
              controls,
              form: props.form,
              valueChange,
              formValues,
              handleSubmit,
              isValid,
              errors,
          };
      },
  });

  function render$4(_ctx, _cache, $props, $setup, $data, $options) {
    const _component_dynamic_input = vue.resolveComponent("dynamic-input");

    return (vue.openBlock(), vue.createBlock("form", {
      class: "dynamic-form",
      novalidate: "",
      id: _ctx.form.id,
      name: _ctx.form.id,
      onSubmit: _cache[1] || (_cache[1] = vue.withModifiers((...args) => (_ctx.handleSubmit(...args)), ["prevent"]))
    }, [
      (vue.openBlock(true), vue.createBlock(vue.Fragment, null, vue.renderList(_ctx.controls, (control) => {
        return (vue.openBlock(), vue.createBlock(_component_dynamic_input, {
          key: control?.name,
          control: control,
          onChanged: _ctx.valueChange
        }, null, 8 /* PROPS */, ["control", "onChanged"]))
      }), 128 /* KEYED_FRAGMENT */))
    ], 40 /* PROPS, HYDRATE_EVENTS */, ["id", "name"]))
  }

  script$4.render = render$4;
  script$4.__file = "src/components/dynamic-form/DynamicForm.vue";

  function createDynamicForms(options) {
      const vdf = {
          options,
          install(app) {
              const self = this;
              app.component('dynamic-form', script$4);
              app.provide(dynamicFormsSymbol, self);
          },
      };
      return vdf;
  }

  exports.EmailInput = EmailInput;
  exports.FormControl = FormControl;
  exports.FormValidation = FormValidation;
  exports.InputBase = InputBase;
  exports.PasswordInput = PasswordInput;
  exports.SelectInput = SelectInput;
  exports.TextAreaInput = TextAreaInput;
  exports.TextInput = TextInput;
  exports.createDynamicForms = createDynamicForms;
  exports.dynamicFormsSymbol = dynamicFormsSymbol;
  exports.useDynamicForms = useDynamicForms;

  return exports;

}({}, Vue));
