import { App } from 'vue';
import { InjectionKey } from 'vue';

export declare function createDynamicForms(options?: DynamicFormsOptions): DynamicFormsPlugin;

export declare interface DynamicFormsOptions {
    theme?: string;
}

export declare interface DynamicFormsPlugin {
    options?: DynamicFormsOptions;
    install(app: App): void;
}

export declare const dynamicFormsSymbol: InjectionKey<DynamicFormsPlugin>;

export declare class EmailInput extends InputBase<string> {
    type: string;
}

export declare class FormControl<T> extends InputBase<T> {
    valid: boolean;
    invalid: boolean;
    dirty: boolean;
    touched: boolean;
    errors: ValidationErrors | null;
    constructor(options?: {
        value?: T | undefined;
        name?: string;
        label?: string;
        disabled?: boolean;
        order?: number;
        type?: string;
        customClass?: string;
        validations?: FormValidation[];
        options?: {
            key: string;
            value: string;
            disabled?: boolean;
        }[];
        cols?: number;
        rows?: number;
    });
}

export declare class FormValidation {
    validator: ValidatorFn;
    text: string;
    constructor(validator: ValidatorFn, text: string);
}

export declare class InputBase<T> {
    value: T | undefined;
    name: string;
    label: string;
    disabled?: boolean;
    order?: number;
    type: string;
    customClass?: string;
    placeholder?: string;
    validations?: FormValidation[];
    options?: {
        key: string;
        value: string;
        disabled?: boolean;
    }[];
    cols?: number;
    rows?: number;
    constructor(options?: {
        value?: T | undefined;
        name?: string;
        label?: string;
        disabled?: boolean;
        order?: number;
        type?: string;
        customClass?: string;
        placeholder?: string;
        validations?: FormValidation[];
        options?: {
            key: string;
            value: string;
            disabled?: boolean;
        }[];
        cols?: number;
        rows?: number;
    });
}

export declare class PasswordInput extends InputBase<string> {
    type: string;
}

export declare class SelectInput<T> extends InputBase<T> {
    type: string;
}

export declare class TextAreaInput extends InputBase<string> {
    type: string;
}

export declare class TextInput extends InputBase<string> {
    type: string;
}

export declare function useDynamicForms(): DynamicFormsPlugin;

declare type ValidationErrors = {
    [key: string]: any;
};

declare interface ValidatorFn {
    (control: FormControl<any> | undefined): ValidationErrors | null;
}

export { }
