declare type ValidationErrors = {
    [key: string]: any;
};
interface ValidatorFn {
    (control: FormControl<any> | undefined): ValidationErrors | null;
}
export declare class FormValidation {
    validator: ValidatorFn;
    text: string;
    constructor(validator: ValidatorFn, text: string);
}
export declare class InputBase<T> {
    value: T | undefined;
    name: string;
    label: string;
    disabled?: boolean;
    order?: number;
    type: string;
    customClass?: string;
    placeholder?: string;
    validations?: FormValidation[];
    options?: {
        key: string;
        value: string;
        disabled?: boolean;
    }[];
    cols?: number;
    rows?: number;
    constructor(options?: {
        value?: T | undefined;
        name?: string;
        label?: string;
        disabled?: boolean;
        order?: number;
        type?: string;
        customClass?: string;
        placeholder?: string;
        validations?: FormValidation[];
        options?: {
            key: string;
            value: string;
            disabled?: boolean;
        }[];
        cols?: number;
        rows?: number;
    });
}
export declare class TextInput extends InputBase<string> {
    type: string;
}
export declare class EmailInput extends InputBase<string> {
    type: string;
}
export declare class PasswordInput extends InputBase<string> {
    type: string;
}
export declare class SelectInput<T> extends InputBase<T> {
    type: string;
}
export declare class TextAreaInput extends InputBase<string> {
    type: string;
}
export declare class FormControl<T> extends InputBase<T> {
    valid: boolean;
    invalid: boolean;
    dirty: boolean;
    touched: boolean;
    errors: ValidationErrors | null;
    constructor(options?: {
        value?: T | undefined;
        name?: string;
        label?: string;
        disabled?: boolean;
        order?: number;
        type?: string;
        customClass?: string;
        validations?: FormValidation[];
        options?: {
            key: string;
            value: string;
            disabled?: boolean;
        }[];
        cols?: number;
        rows?: number;
    });
}
export {};
//# sourceMappingURL=models.d.ts.map